@@ -35,6 +35,7 @@ def _bootstrap_environment():
     # Install requirements
     ssh.run("apt-get install -q -y pypy libpq-dev libffi-dev rubygems")
     ssh.run("apt-get install -q -y git dpkg-sig reprepro nginx")
+    ssh.run("apt-get install node-less")
     ssh.run(
         "curl https://bitbucket.org/pypa/setuptools/raw/bootstrap/ez_setup.py "
         "| pypy"
@@ -75,6 +76,7 @@ def _build_package():
     ssh.run("mkdir -p /opt/warehouse")
     ssh.run("mkdir -p /opt/warehouse/etc")
     ssh.run("mkdir -p /opt/warehouse/var/www")
+    ssh.run("mkdir -p /opt/warehouse/var/run")
 
     # Create a virtual environment
     ssh.run("virtualenv -p pypy /opt/warehouse")
@@ -86,12 +88,55 @@ def _build_package():
         "warehouse"
     )
 
+    # Write out Build Time Configuration
+    ssh.put(
+        io.BytesIO(textwrap.dedent("""
+            assets:
+                directory: /opt/warehouse/var/www/static
+        """).strip() + "\n"),
+        "/opt/warehouse/etc/warehouse.build.yml",
+    )
+
+    # Run the collectstatic command
+    ssh.run(
+        "/opt/warehouse/bin/warehouse "
+        "-c /opt/warehouse/etc/warehouse.build.yml collectstatic"
+    )
+
     # Get the installed version of warehouse
     version = ssh.run(
         "/opt/warehouse/bin/python -c "
         "'import warehouse; print(warehouse.__version__)'",
     ).strip()
 
+    # Make a file that will be ran after the install of the package completes
+    ssh.put(
+        io.BytesIO(textwrap.dedent("""
+            #!/bin/sh
+            set -e
+
+            # Create our user if we need too
+            if ! getent group warehouse > /dev/null 2>&1; then
+                useradd -d /opt/warehouse -M -r warehouse
+            fi
+
+            # Fix up directory permissions
+            chown -Rf root:warehouse /opt/warehouse
+
+        """).strip() + "\n"),
+        "/opt/warehouse.postinst",
+    )
+
+    ssh.run("chmod +x /opt/warehouse.postinst")
+
+    # Fix permissions
+    ssh.run("find /opt/warehouse -type d -exec chmod 750 {} \\;")
+    ssh.run("find /opt/warehouse -type f -exec chmod 640 {} \\;")
+    ssh.run("chmod -R 750 /opt/warehouse/bin")
+    ssh.run("chmod 777 /opt/warehouse/var/run")
+    ssh.run("find /opt/warehouse/var/www -type d -exec chmod 755 {} \\;")
+    ssh.run("find /opt/warehouse/var/www -type f -exec chmod 644 {} \\;")
+
     # Make a directory for our built packages to go in
     ssh.run("mkdir -p ~/packages")
 
@@ -100,6 +145,7 @@ def _build_package():
             "fpm -t deb -s dir -n warehouse -v {} --iteration 1 "
             "-m 'Donald Stufft <donald@stufft.io>' "
             "-d pypy -d libpq5 -d libffi6 "
+            "--after-install /opt/warehouse.postinst "
             "/opt/warehouse".format(version),
         )
 
